@@ -167,7 +167,7 @@ cp1251_apply_all_case_fold(OnigCaseFoldType flag,
 			       OnigApplyAllCaseFoldFunc f, void* arg, OnigEncoding enc ARG_UNUSED)
 {
   return onigenc_apply_all_case_fold_with_map(
-             sizeof(CaseFoldMap)/sizeof(OnigPairCaseFoldCodes), CaseFoldMap, 0,
+             numberof(CaseFoldMap), CaseFoldMap, 0,
              flag, f, arg);
 }
 
@@ -176,7 +176,7 @@ cp1251_get_case_fold_codes_by_str(OnigCaseFoldType flag,
     const OnigUChar* p, const OnigUChar* end, OnigCaseFoldCodeItem items[], OnigEncoding enc ARG_UNUSED)
 {
   return onigenc_get_case_fold_codes_by_str_with_map(
-	     sizeof(CaseFoldMap)/sizeof(OnigPairCaseFoldCodes), CaseFoldMap, 0,
+	     numberof(CaseFoldMap), CaseFoldMap, 0,
 	     flag, p, end, items);
 }
 
@@ -196,7 +196,9 @@ OnigEncodingDefine(windows_1251, Windows_1251) = {
   cp1251_is_code_ctype,
   onigenc_not_support_get_ctype_code_range,
   onigenc_single_byte_left_adjust_char_head,
-  onigenc_always_true_is_allowed_reverse_match
+  onigenc_always_true_is_allowed_reverse_match,
+  0,
+  ONIGENC_FLAG_NONE,
 };
 /*
  * Name: windows-1251
@@ -205,4 +207,4 @@ OnigEncodingDefine(windows_1251, Windows_1251) = {
  * Link: http://www.microsoft.com/globaldev/reference/sbcs/1251.mspx
  * Link: http://en.wikipedia.org/wiki/Windows-1251
  */
-ENC_ALIAS("CP1251", "Windows-1251");
+ENC_ALIAS("CP1251", "Windows-1251")
