@@ -2,13 +2,13 @@
 
   readline.c - GNU Readline module
 
-  $Author: nagachika $
+  $Author: akr $
   created at: Wed Jan 20 13:59:32 JST 1999
 
   Copyright (C) 1997-2008  Shugo Maeda
   Copyright (C) 2008-2013  Kouji Takao
 
-  $Id: readline.c 45772 2014-05-01 11:55:03Z nagachika $
+  $Id: readline.c 48447 2014-11-15 11:49:06Z akr $
 
   Contact:
    - Kouji Takao <kouji dot takao at gmail dot com> (current maintainer)
@@ -33,7 +33,7 @@
 #include <editline/readline.h>
 #endif
 
-#include "ruby/ruby.h"
+#include "internal.h"
 #include "ruby/io.h"
 #include "ruby/thread.h"
 
@@ -688,12 +688,7 @@ readline_s_delete_text(int argc, VALUE *argv, VALUE self)
         char *p, *ptr = rl_line_buffer;
         long beg = 0, len = strlen(rl_line_buffer);
         struct RString fakestr;
-        VALUE str = (VALUE)&fakestr;
-
-        fakestr.basic.flags = T_STRING | RSTRING_NOEMBED;
-        fakestr.as.heap.ptr = ptr;
-        fakestr.as.heap.len = len;
-        rb_enc_associate(str, rb_locale_encoding());
+        VALUE str = rb_setup_fake_str(&fakestr, ptr, len, rb_locale_encoding());
         OBJ_FREEZE(str);
         if (argc == 2) {
             beg = NUM2LONG(argv[0]);
@@ -1790,7 +1785,7 @@ username_completion_proc_call(VALUE self, VALUE str)
 }
 
 void
-Init_readline()
+Init_readline(void)
 {
     VALUE history, fcomp, ucomp, version;
 
