@@ -3,7 +3,6 @@ require 'test/unit'
 require 'net/http'
 require 'stringio'
 require_relative 'utils'
-require_relative '../../ruby/envutil'
 
 class TestNetHTTP < Test::Unit::TestCase
 
@@ -277,6 +276,7 @@ module TestNetHTTP_version_1_1_methods
       end
     }
     assert_equal 1, i
+    @log_tester = nil # server may encount ECONNRESET
   end
 
   def test_get__implicit_start
@@ -397,6 +397,7 @@ module TestNetHTTP_version_1_1_methods
   def test_timeout_during_HTTP_session
     bug4246 = "expected the HTTP session to have timed out but have not. c.f. [ruby-core:34203]"
 
+    th = nil
     # listen for connections... but deliberately do not read
     TCPServer.open('localhost', 0) {|server|
       port = server.addr[1]
@@ -412,6 +413,9 @@ module TestNetHTTP_version_1_1_methods
       end
       assert th.join(10), bug4246
     }
+  ensure
+    th.kill
+    th.join
   end
 end
 
@@ -559,10 +563,10 @@ module TestNetHTTP_version_1_2_methods
   end
 
   def _test_request__uri_host(http)
-    uri = URI 'http://example/'
+    uri = URI 'http://other.example/'
 
     req = Net::HTTP::Get.new(uri)
-    req['host'] = 'other.example'
+    req['host'] = 'example'
 
     res = http.request(req)
 
@@ -574,7 +578,6 @@ module TestNetHTTP_version_1_2_methods
   def test_send_request
     start {|http|
       _test_send_request__GET http
-      _test_send_request__HEAD http
       _test_send_request__POST http
     }
   end
@@ -589,16 +592,6 @@ module TestNetHTTP_version_1_2_methods
     assert_equal $test_net_http_data, res.body
   end
 
-  def _test_send_request__HEAD(http)
-    res = http.send_request('HEAD', '/')
-    assert_kind_of Net::HTTPResponse, res
-    unless self.is_a?(TestNetHTTP_v1_2_chunked)
-      assert_not_nil res['content-length']
-      assert_equal $test_net_http_data.size, res['content-length'].to_i
-    end
-    assert_nil res.body
-  end
-
   def _test_send_request__POST(http)
     data = 'aaabbb cc ddddddddddd lkjoiu4j3qlkuoa'
     res = http.send_request('POST', '/', data, 'content-type' => 'application/x-www-form-urlencoded')
@@ -822,22 +815,6 @@ class TestNetHTTPContinue < Test::Unit::TestCase
     assert_not_match(/HTTP\/1.1 100 continue/, @debug.string)
   end
 
-  def test_expect_continue_error_before_body
-    @log_tester = nil
-    mount_proc {|req, res|
-      raise WEBrick::HTTPStatus::Forbidden
-    }
-    start {|http|
-      uheader = {'content-length' => '5', 'expect' => '100-continue'}
-      http.continue_timeout = 1 # allow the server to respond before sending
-      http.request_post('/continue', 'data', uheader) {|res|
-        assert_equal(res.code, '403')
-      }
-    }
-    assert_match(/Expect: 100-continue/, @debug.string)
-    assert_not_match(/HTTP\/1.1 100 continue/, @debug.string)
-  end
-
   def test_expect_continue_error_while_waiting
     mount_proc {|req, res|
       res.status = 501
