@@ -1,3 +1,4 @@
+# coding: US-ASCII
 require 'test/unit'
 require 'stringio'
 
@@ -12,7 +13,7 @@ class TestParse < Test::Unit::TestCase
   end
 
   def test_else_without_rescue
-    x = eval <<-END
+    x = eval <<-END, nil, __FILE__, __LINE__+1
       begin
       else
         42
@@ -20,10 +21,10 @@ class TestParse < Test::Unit::TestCase
     END
     assert_equal(42, x)
   end
-  
+
   def test_alias_backref
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         alias $foo $1
       END
     end
@@ -36,7 +37,7 @@ class TestParse < Test::Unit::TestCase
     a = false
     b = c = d = true
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a &&= t.foo 42
         b &&= t.foo 42
         c &&= t.foo nil
@@ -51,7 +52,7 @@ class TestParse < Test::Unit::TestCase
 
     a = [nil, nil, true, true]
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a[0] ||= t.foo 42
         a[1] &&= t.foo 42
         a[2] ||= t.foo 42
@@ -67,7 +68,7 @@ class TestParse < Test::Unit::TestCase
     o.foo = o.Foo = o::baz = nil
     o.bar = o.Bar = o::qux = 1
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         o.foo ||= t.foo 42
         o.bar &&= t.foo 42
         o.Foo ||= t.foo 42
@@ -81,7 +82,7 @@ class TestParse < Test::Unit::TestCase
     assert_equal([42, 42], [o::baz, o::qux])
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         $1 ||= t.foo 42
       END
     end
@@ -90,7 +91,7 @@ class TestParse < Test::Unit::TestCase
 
     a = b = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a = t.bar "foo" do
           "bar"
         end.gsub "ob", "OB"
@@ -104,16 +105,16 @@ class TestParse < Test::Unit::TestCase
 
     a = nil
     assert_nothing_raised do
-      t.instance_eval <<-END
-        a = bar "foo" { "bar" }
+      t.instance_eval <<-END, __FILE__, __LINE__+1
+        a = bar "foo" do "bar" end
       END
     end
     assert_equal("foobar", a)
 
     a = nil
     assert_nothing_raised do
-      eval <<-END
-        a = t::bar "foo" { "bar" }
+      eval <<-END, nil, __FILE__, __LINE__+1
+        a = t::bar "foo" do "bar" end
       END
     end
     assert_equal("foobar", a)
@@ -122,45 +123,10 @@ class TestParse < Test::Unit::TestCase
       @baz = r + (block_given? ? [yield] : [])
     end
 
-    assert_nothing_raised do
-      t.instance_eval "baz ()"
-    end
-    assert_equal([], t.instance_eval { @baz })
-
     assert_nothing_raised do
       t.instance_eval "baz (1), 2"
     end
     assert_equal([1, 2], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (3, 4)"
-    end
-    assert_equal([3, 4], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (5, &proc{6})"
-    end
-    assert_equal([5, 6], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (7 => 8)"
-    end
-    assert_equal([{ 7 => 8 }], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (9, 10 => 11)"
-    end
-    assert_equal([9, { 10 => 11 }], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (12, 13, 14 => 15)"
-    end
-    assert_equal([12, 13, { 14 => 15 }], t.instance_eval { @baz })
-
-    assert_nothing_raised do
-      t.instance_eval "baz (&proc {16})"
-    end
-    assert_equal([16], t.instance_eval { @baz })
   end
 
   def test_mlhs_node
@@ -171,7 +137,7 @@ class TestParse < Test::Unit::TestCase
     end
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         c::foo, c::bar = 1, 2
         c.Foo, c.Bar = 1, 2
         c::FOO, c::BAR = 1, 2
@@ -184,7 +150,7 @@ class TestParse < Test::Unit::TestCase
 
   def test_dynamic_constant_assignment
     assert_raise(SyntaxError) do
-      Object.new.instance_eval <<-END
+      Object.new.instance_eval <<-END, __FILE__, __LINE__+1
         def foo
           self::FOO, self::BAR = 1, 2
           ::FOO, ::BAR = 1, 2
@@ -193,13 +159,13 @@ class TestParse < Test::Unit::TestCase
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         $1, $2 = 1, 2
       END
     end
 
     assert_raise(SyntaxError) do
-      Object.new.instance_eval <<-END
+      Object.new.instance_eval <<-END, __FILE__, __LINE__+1
         def foo
           ::FOO = 1
         end
@@ -207,16 +173,18 @@ class TestParse < Test::Unit::TestCase
     end
 
     c = Class.new
-    assert_raise(SyntaxError) do
-      eval <<-END
+    assert_nothing_raised(SyntaxError) do
+      eval <<-END, nil, __FILE__, __LINE__+1
+      if false
         c::FOO &= 1
         ::FOO &= 1
+      end
       END
     end
 
     c = Class.new
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         $1 &= 1
       END
     end
@@ -224,13 +192,13 @@ class TestParse < Test::Unit::TestCase
 
   def test_class_module
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         class foo; end
       END
     end
-    
+
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo
           class Foo; end
           module Bar; end
@@ -239,7 +207,7 @@ class TestParse < Test::Unit::TestCase
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         class Foo Bar; end
       END
     end
@@ -250,9 +218,8 @@ class TestParse < Test::Unit::TestCase
     def o.>(x); x; end
     def o./(x); x; end
 
-    a = nil
     assert_nothing_raised do
-      o.instance_eval <<-END
+      o.instance_eval <<-END, __FILE__, __LINE__+1
         undef >, /
       END
     end
@@ -266,7 +233,7 @@ class TestParse < Test::Unit::TestCase
     o.foo = o.Foo = o::baz = nil
     o.bar = o.Bar = o::qux = 1
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         o.foo ||= 42
         o.bar &&= 42
         o.Foo ||= 42
@@ -281,34 +248,21 @@ class TestParse < Test::Unit::TestCase
 
     a = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a = -2.0 ** 2
       END
     end
     assert_equal(-4.0, a)
   end
 
-  def test_open_args
-    o = Object.new
-    def o.foo(*r); r.inject(42, :+); end
-
-    a = nil
-    assert_nothing_raised do
-      o.instance_eval <<-END
-        a = foo ()
-      END
-    end
-    assert_equal(42, a)
-  end
-
   def test_block_variable
     o = Object.new
     def o.foo(*r); yield(*r); end
 
     a = nil
     assert_nothing_raised do
-      eval <<-END
-        o.foo 1 {|; a| a = 42 }
+      eval <<-END, nil, __FILE__, __LINE__+1
+        o.foo 1 do|; a| a = 42 end
       END
     end
     assert_nil(a)
@@ -316,25 +270,25 @@ class TestParse < Test::Unit::TestCase
 
   def test_bad_arg
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo(FOO); end
       END
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo(@foo); end
       END
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo($foo); end
       END
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo(@@foo); end
       END
     end
@@ -343,7 +297,7 @@ class TestParse < Test::Unit::TestCase
     def o.foo(*r); yield(*r); end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         o.foo 1 {|; @a| @a = 42 }
       END
     end
@@ -352,7 +306,7 @@ class TestParse < Test::Unit::TestCase
   def test_do_lambda
     a = b = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a = -> do
           b = 42
         end
@@ -368,7 +322,7 @@ class TestParse < Test::Unit::TestCase
 
     a = b = nil
     assert_nothing_raised do
-      o.instance_eval <<-END
+      o.instance_eval <<-END, __FILE__, __LINE__+1
         a = foo 1 do 42 end.to_s
         b = foo 1 do 42 end::to_s
       END
@@ -380,7 +334,7 @@ class TestParse < Test::Unit::TestCase
   def test_call_method
     a = b = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a = proc {|x| x + "bar" }.("foo")
         b = proc {|x| x + "bar" }::("foo")
       END
@@ -398,7 +352,7 @@ class TestParse < Test::Unit::TestCase
   def test_words
     assert_equal([], %W( ))
   end
-  
+
   def test_dstr
     @@foo = 1
     assert_equal("foo 1 bar", "foo #@@foo bar")
@@ -406,6 +360,17 @@ class TestParse < Test::Unit::TestCase
     assert_equal("foo 1 bar", "foo #$1 bar")
   end
 
+  def test_dstr_disallowed_variable
+    bug8375 = '[ruby-core:54885] [Bug #8375]'
+    %w[@ @1 @@. @@ @@1 @@. $ $%].each do |src|
+      src = '#'+src+' '
+      str = assert_nothing_raised(SyntaxError, "#{bug8375} #{src.dump}") do
+        break eval('"'+src+'"')
+      end
+      assert_equal(src, str, bug8375)
+    end
+  end
+
   def test_dsym
     assert_nothing_raised { eval(':""') }
   end
@@ -414,7 +379,7 @@ class TestParse < Test::Unit::TestCase
     o = Object.new
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def o.foo(a=42,*r,z,&b); b.call(r.inject(a*1000+z*100, :+)); end
       END
     end
@@ -425,7 +390,7 @@ class TestParse < Test::Unit::TestCase
     assert_raise(ArgumentError) { o.foo() }
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def o.foo(a=42,z,&b); b.call(a*1000+z*100); end
       END
     end
@@ -434,7 +399,7 @@ class TestParse < Test::Unit::TestCase
     assert_raise(ArgumentError) { o.foo() }
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def o.foo(*r,z,&b); b.call(r.inject(z*100, :+)); end
       END
     end
@@ -446,19 +411,19 @@ class TestParse < Test::Unit::TestCase
 
   def test_duplicate_argument
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         1.times {|&b?| }
       END
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         1.times {|a, a|}
       END
     end
 
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo(a, a); end
       END
     end
@@ -466,7 +431,7 @@ class TestParse < Test::Unit::TestCase
 
   def test_define_singleton_error
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def ("foo").foo; end
       END
     end
@@ -476,17 +441,17 @@ class TestParse < Test::Unit::TestCase
     t = Object.new
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def t.`(x); "foo" + x + "bar"; end
       END
     end
     a = b = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         a = t.` "zzz"
         1.times {|;z| t.` ("zzz") }
       END
-      t.instance_eval <<-END
+      t.instance_eval <<-END, __FILE__, __LINE__+1
         b = `zzz`
       END
     end
@@ -539,14 +504,16 @@ class TestParse < Test::Unit::TestCase
   end
 
   def test_symbol
-    assert_raise(SyntaxError) do
-      eval ":'foo\0bar'"
+    bug = '[ruby-dev:41447]'
+    sym = "foo\0bar".to_sym
+    assert_nothing_raised(SyntaxError, bug) do
+      assert_equal(sym, eval(":'foo\0bar'"))
     end
-    assert_raise(SyntaxError) do
-      eval ':"foo\u0000bar"'
+    assert_nothing_raised(SyntaxError, bug) do
+      assert_equal(sym, eval(':"foo\u0000bar"'))
     end
-    assert_raise(SyntaxError) do
-      eval ':"foo\u{0}bar"'
+    assert_nothing_raised(SyntaxError, bug) do
+      assert_equal(sym, eval(':"foo\u{0}bar"'))
     end
     assert_raise(SyntaxError) do
       eval ':"foo\u{}bar"'
@@ -555,7 +522,7 @@ class TestParse < Test::Unit::TestCase
 
   def test_parse_string
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
 /
       END
     end
@@ -565,22 +532,23 @@ class TestParse < Test::Unit::TestCase
     x = nil
 
     assert_raise(SyntaxError) do
-      eval %q(
-<<FOO
+      eval %Q(
+<\<FOO
       )
     end
 
-    assert_raise(SyntaxError) do
-      eval %q(
+    assert_nothing_raised(SyntaxError) do
+      x = eval %q(
 <<FOO
 #$
 FOO
       )
     end
+    assert_equal "\#$\n", x
 
     assert_raise(SyntaxError) do
-      eval %q(
-<<"
+      eval %Q(
+<\<\"
       )
     end
 
@@ -596,17 +564,18 @@ FOO
       )
     end
 
-    assert_raise(SyntaxError) do
-      eval %q(
+    assert_nothing_raised(SyntaxError) do
+      x = eval %q(
 <<FOO
 #$
 foo
 FOO
       )
     end
+    assert_equal "\#$\nfoo\n", x
 
     assert_nothing_raised do
-      eval "x = <<FOO\r\n1\r\nFOO"
+      eval "x = <<""FOO\r\n1\r\nFOO"
     end
     assert_equal("1\n", x)
   end
@@ -614,7 +583,7 @@ FOO
   def test_magic_comment
     x = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
 # coding = utf-8
 x = __ENCODING__
       END
@@ -622,7 +591,7 @@ x = __ENCODING__
     assert_equal(Encoding.find("UTF-8"), x)
 
     assert_raise(ArgumentError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
 # coding = foobarbazquxquux_dummy_enconding
 x = __ENCODING__
       END
@@ -641,7 +610,7 @@ x = __ENCODING__
   def test_dot_in_next_line
     x = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         x = 1
         .to_s
       END
@@ -657,7 +626,7 @@ x = __ENCODING__
 
   def test_embedded_rd
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
 =begin
       END
     end
@@ -681,15 +650,20 @@ x = __ENCODING__
 
   def test_invalid_instance_variable
     assert_raise(SyntaxError) { eval('@#') }
+    assert_raise(SyntaxError) { eval('@') }
   end
 
   def test_invalid_class_variable
     assert_raise(SyntaxError) { eval('@@1') }
+    assert_raise(SyntaxError) { eval('@@') }
   end
 
   def test_invalid_char
+    bug10117 = '[ruby-core:64243] [Bug #10117]'
+    invalid_char = /Invalid char `\\x01'/
     x = 1
-    assert_equal(1, eval("\x01x"))
+    assert_in_out_err(%W"-e \x01x", "", [], invalid_char, bug10117)
+    assert_syntax_error("\x01x", invalid_char, bug10117)
     assert_equal(nil, eval("\x04x"))
   end
 
@@ -721,7 +695,7 @@ x = __ENCODING__
       eval %q(__ENCODING__ = 1)
     end
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         def foo
           FOO = 1
         end
@@ -731,7 +705,7 @@ x = __ENCODING__
 
   def test_block_dup
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         foo(&proc{}) {}
       END
     end
@@ -739,7 +713,7 @@ x = __ENCODING__
 
   def test_set_backref
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         $& = 1
       END
     end
@@ -752,7 +726,7 @@ x = __ENCODING__
     end
     r = nil
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         o[&proc{|x| r = x }] = 1
       END
     end
@@ -782,25 +756,19 @@ x = __ENCODING__
       eval %q(1; next; 2)
     end
 
-    o = Object.new
-    assert_nothing_raised do
-      eval <<-END
-        x = def o.foo; end
-      END
-    end
-    assert_equal($stderr.string.lines.to_a.size, 14)
+    assert_equal(13, $stderr.string.lines.to_a.size)
     $stderr = stderr
   end
 
   def test_assign_in_conditional
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         (x, y = 1, 2) ? 1 : 2
       END
     end
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         if @x = true
           1
         else
@@ -812,33 +780,33 @@ x = __ENCODING__
 
   def test_literal_in_conditional
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         "foo" ? 1 : 2
       END
     end
 
     assert_nothing_raised do
       x = "bar"
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         /foo#{x}baz/ ? 1 : 2
       END
     end
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         (true..false) ? 1 : 2
       END
     end
 
     assert_nothing_raised do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         ("foo".."bar") ? 1 : 2
       END
     end
 
     assert_nothing_raised do
       x = "bar"
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         :"foo#{"x"}baz" ? 1 : 2
       END
     end
@@ -846,7 +814,7 @@ x = __ENCODING__
 
   def test_no_blockarg
     assert_raise(SyntaxError) do
-      eval <<-END
+      eval <<-END, nil, __FILE__, __LINE__+1
         yield(&:+)
       END
     end
@@ -862,7 +830,7 @@ x = __ENCODING__
   def test_all_symbols
     x = Symbol.all_symbols
     assert_kind_of(Array, x)
-    assert(x.all? {|s| s.is_a?(Symbol) })
+    assert_empty(x.reject {|s| s.is_a?(Symbol) })
   end
 
   def test_is_class_id
@@ -871,4 +839,42 @@ x = __ENCODING__
       c.instance_eval { remove_class_variable(:@var) }
     end
   end
+
+  def test_method_block_location
+    bug5614 = '[ruby-core:40936]'
+    expected = nil
+    e = assert_raise(NoMethodError) do
+      1.times do
+        expected = __LINE__+1
+      end.print do
+        #
+      end
+    end
+    actual = e.backtrace.first[/\A#{Regexp.quote(__FILE__)}:(\d+):/o, 1].to_i
+    assert_equal(expected, actual, bug5614)
+  end
+
+  def test_shadowing_variable
+    assert_warning(/shadowing outer local variable/) {eval("a=1; tap {|a|}")}
+    a = "\u{3042}"
+    assert_warning(/#{a}/o) {eval("#{a}=1; tap {|#{a}|}")}
+  end
+
+  def test_unused_variable
+    o = Object.new
+    assert_warning(/assigned but unused variable/) {o.instance_eval("def foo; a=1; nil; end")}
+    a = "\u{3042}"
+    assert_warning(/#{a}/) {o.instance_eval("def foo; #{a}=1; nil; end")}
+  end
+
+  def test_named_capture_conflict
+    a = 1
+    assert_warning(/named capture conflict/) {eval("a = 1; /(?<a>)/ =~ ''")}
+    a = "\u{3042}"
+    assert_warning(/#{a}/) {eval("#{a} = 1; /(?<#{a}>)/ =~ ''")}
+  end
+
+  def test_past_scope_variable
+    assert_warning(/past scope/) {catch {|tag| eval("BEGIN{throw tag}; tap {a = 1}; a")}}
+  end
 end
